@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/common/commands.py
 ##
-## Copyright (C) 2006-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2006-2007 Tomasz Melcer <liori AT exroot.org>
 ## Copyright (C) 2007 Jean-Marie Traissard <jim AT lapin.org>
 ## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
@@ -22,15 +22,19 @@
 ## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 ##
 
-import xmpp
+import nbxmpp
 import helpers
 import dataforms
 import gajim
+from common.connection_handlers_events import MessageOutgoingEvent
+
+import logging
+log = logging.getLogger('gajim.c.commands')
 
 class AdHocCommand:
     commandnode = 'command'
     commandname = 'The Command'
-    commandfeatures = (xmpp.NS_DATA,)
+    commandfeatures = (nbxmpp.NS_DATA,)
 
     @staticmethod
     def isVisibleFor(samejid):
@@ -53,10 +57,10 @@ class AdHocCommand:
         assert status in ('executing', 'completed', 'canceled')
 
         response = request.buildReply('result')
-        cmd = response.addChild('command', namespace=xmpp.NS_COMMANDS, attrs={
-                'sessionid': self.sessionid,
-                'node': self.commandnode,
-                'status': status})
+        cmd = response.getTag('command', namespace=nbxmpp.NS_COMMANDS)
+        cmd.setAttr('sessionid', self.sessionid)
+        cmd.setAttr('node', self.commandnode)
+        cmd.setAttr('status', status)
         if defaultaction is not None or actions is not None:
             if defaultaction is not None:
                 assert defaultaction in ('cancel', 'execute', 'prev', 'next',
@@ -69,8 +73,8 @@ class AdHocCommand:
         return response, cmd
 
     def badRequest(self, stanza):
-        self.connection.connection.send(xmpp.Error(stanza, xmpp.NS_STANZAS + \
-                ' bad-request'))
+        self.connection.connection.send(nbxmpp.Error(stanza,
+            nbxmpp.NS_STANZAS + ' bad-request'))
 
     def cancel(self, request):
         response = self.buildResponse(request, status = 'canceled')[0]
@@ -274,13 +278,18 @@ class ForwardMessagesCommand(AdHocCommand):
     def execute(self, request):
         account = self.connection.name
         # Forward messages
-        events = gajim.events.get_events(account, types=['chat', 'normal'])
+        events = gajim.events.get_events(account, types=['chat', 'normal',
+            'printed_chat'])
         j, resource = gajim.get_room_and_nick_from_fjid(self.jid)
         for jid in events:
             for event in events[jid]:
-                self.connection.send_message(j, event.parameters[0], '',
-                        type_=event.type_, subject=event.parameters[1],
-                        resource=resource, forward_from=jid, delayed=event.time_)
+                ev_typ = event.type_
+                if ev_typ == 'printed_chat':
+                    ev_typ = 'chat'
+                gajim.nec.push_outgoing_event(MessageOutgoingEvent(None,
+                    account=account, jid=j, message=event.parameters[0],
+                    type_=ev_typ, subject=event.parameters[1],
+                    resource=resource, forward_from=jid, delayed=event.time_))
 
         # Inform other client of completion
         response, cmd = self.buildResponse(request, status = 'completed')
@@ -308,10 +317,14 @@ class FwdMsgThenDisconnectCommand(AdHocCommand):
         j, resource = gajim.get_room_and_nick_from_fjid(self.jid)
         for jid in events:
             for event in events[jid]:
-                self.connection.send_message(j, event.parameters[0], '',
-                    type_=event.type_, subject=event.parameters[1],
+                ev_typ = event.type_
+                if ev_typ == 'printed_chat':
+                    ev_typ = 'chat'
+                gajim.nec.push_outgoing_event(MessageOutgoingEvent(None,
+                    account=account, jid=j, message=event.parameters[0],
+                    type_=ev_typ, subject=event.parameters[1],
                     resource=resource, forward_from=jid, delayed=event.time_,
-                    now=True)
+                    now=True))
 
         response, cmd = self.buildResponse(request, status = 'completed')
         cmd.addChild('note', {}, _('The status has been changed.'))
@@ -347,14 +360,14 @@ class ConnectionCommands:
         """
         Test if the bare jid given is the same as our bare jid
         """
-        return xmpp.JID(jid).getStripped() == self.getOurBareJID()
+        return nbxmpp.JID(jid).getStripped() == self.getOurBareJID()
 
     def commandListQuery(self, con, iq_obj):
         iq = iq_obj.buildReply('result')
         jid = helpers.get_full_jid_from_iq(iq_obj)
         q = iq.getTag('query')
         # buildReply don't copy the node attribute. Re-add it
-        q.setAttr('node', xmpp.NS_COMMANDS)
+        q.setAttr('node', nbxmpp.NS_COMMANDS)
 
         for node, cmd in self.__commands.iteritems():
             if cmd.isVisibleFor(self.isSameJID(jid)):
@@ -371,7 +384,11 @@ class ConnectionCommands:
         Send disco#info result for query for command (JEP-0050, example 6.).
         Return True if the result was sent, False if not
         """
-        jid = helpers.get_full_jid_from_iq(iq_obj)
+        try:
+            jid = helpers.get_full_jid_from_iq(iq_obj)
+        except helpers.InvalidFormat:
+            log.warn('Invalid JID: %s, ignoring it' % iq_obj.getFrom())
+            return
         node = iq_obj.getTagAttr('query', 'node')
 
         if node not in self.__commands: return False
@@ -383,7 +400,7 @@ class ConnectionCommands:
             q.addChild('identity', attrs = {'type': 'command-node',
                     'category': 'automation',
                     'name': cmd.commandname})
-            q.addChild('feature', attrs = {'var': xmpp.NS_COMMANDS})
+            q.addChild('feature', attrs = {'var': nbxmpp.NS_COMMANDS})
             for feature in cmd.commandfeatures:
                 q.addChild('feature', attrs = {'var': feature})
 
@@ -425,8 +442,8 @@ class ConnectionCommands:
             # and command exist
             if node not in self.__commands.keys():
                 self.connection.send(
-                        xmpp.Error(iq_obj, xmpp.NS_STANZAS + ' item-not-found'))
-                raise xmpp.NodeProcessed
+                    nbxmpp.Error(iq_obj, nbxmpp.NS_STANZAS + ' item-not-found'))
+                raise nbxmpp.NodeProcessed
 
             newcmd = self.__commands[node]
             if not newcmd.isVisibleFor(self.isSameJID(jid)):
@@ -440,7 +457,7 @@ class ConnectionCommands:
             rc = obj.execute(iq_obj)
             if rc:
                 self.__sessions[(jid, sessionid, node)] = obj
-            raise xmpp.NodeProcessed
+            raise nbxmpp.NodeProcessed
         else:
             # the command is already running, check for it
             magictuple = (jid, sessionid, node)
@@ -475,4 +492,4 @@ class ConnectionCommands:
             if not rc:
                 del self.__sessions[magictuple]
 
-            raise xmpp.NodeProcessed
+            raise nbxmpp.NodeProcessed
