@@ -54,10 +54,8 @@ from common.pubsub import ConnectionPubSub
 from common.protocol.caps import ConnectionCaps
 from common.protocol.bytestream import ConnectionSocks5Bytestream
 from common.protocol.bytestream import ConnectionIBBytestream
-from common.message_archiving import ConnectionArchive
-from common.message_archiving import ARCHIVING_COLLECTIONS_ARRIVED
-from common.message_archiving import ARCHIVING_COLLECTION_ARRIVED
-from common.message_archiving import ARCHIVING_MODIFICATIONS_ARRIVED
+from common.message_archiving import ConnectionArchive136
+from common.message_archiving import ConnectionArchive313
 from common.connection_handlers_events import *
 
 from common import ged
@@ -464,6 +462,7 @@ class ConnectionVcard:
 
         if id_ not in self.awaiting_answers:
             return
+
         if self.awaiting_answers[id_][0] == VCARD_PUBLISHED:
             if iq_obj.getType() == 'result':
                 vcard_iq = self.awaiting_answers[id_][1]
@@ -482,6 +481,7 @@ class ConnectionVcard:
                 if self.vcard_sha != new_sha and gajim.SHOW_LIST[
                 self.connected] != 'invisible':
                     if not self.connection or self.connected < 2:
+                        del self.awaiting_answers[id_]
                         return
                     self.vcard_sha = new_sha
                     sshow = helpers.get_xmpp_show(gajim.SHOW_LIST[
@@ -495,6 +495,7 @@ class ConnectionVcard:
             elif iq_obj.getType() == 'error':
                 gajim.nec.push_incoming_event(VcardNotPublishedEvent(None,
                     conn=self))
+            del self.awaiting_answers[id_]
         elif self.awaiting_answers[id_][0] == VCARD_ARRIVED:
             # If vcard is empty, we send to the interface an empty vcard so that
             # it knows it arrived
@@ -510,17 +511,19 @@ class ConnectionVcard:
                 if id_ in self.groupchat_jids:
                     frm = self.groupchat_jids[id_]
                     del self.groupchat_jids[id_]
-                if frm and frm != our_jid:
+                if frm:
                     # Write an empty file
                     self._save_vcard_to_hd(frm, '')
                 jid, resource = gajim.get_room_and_nick_from_fjid(frm)
                 vcard = {'jid': jid, 'resource': resource}
                 gajim.nec.push_incoming_event(VcardReceivedEvent(None,
                     conn=self, vcard_dict=vcard))
+            del self.awaiting_answers[id_]
         elif self.awaiting_answers[id_][0] == AGENT_REMOVED:
             jid = self.awaiting_answers[id_][1]
             gajim.nec.push_incoming_event(AgentRemovedEvent(None, conn=self,
                 agent=jid))
+            del self.awaiting_answers[id_]
         elif self.awaiting_answers[id_][0] == METACONTACTS_ARRIVED:
             if not self.connection:
                 return
@@ -531,7 +534,9 @@ class ConnectionVcard:
                 if iq_obj.getErrorCode() not in ('403', '406', '404'):
                     self.private_storage_supported = False
             self.get_roster_delimiter()
+            del self.awaiting_answers[id_]
         elif self.awaiting_answers[id_][0] == DELIMITER_ARRIVED:
+            del self.awaiting_answers[id_]
             if not self.connection:
                 return
             if iq_obj.getType() == 'result':
@@ -566,35 +571,20 @@ class ConnectionVcard:
                 gajim.nec.push_incoming_event(RosterReceivedEvent(None,
                     conn=self))
             gobject.timeout_add_seconds(10, self.discover_servers)
+            del self.awaiting_answers[id_]
         elif self.awaiting_answers[id_][0] == PRIVACY_ARRIVED:
+            del self.awaiting_answers[id_]
             if iq_obj.getType() != 'error':
-                self.privacy_rules_supported = True
                 self.get_privacy_list('block')
+                # Ask metacontacts before roster
+                self.get_metacontacts()
             else:
-                if self.blocking_supported:
-                    iq = nbxmpp.Iq('get', xmlns='')
-                    query = iq.setQuery(name='blocklist')
-                    query.setNamespace(nbxmpp.NS_BLOCKING)
-                    id2_ = self.connection.getAnID()
-                    iq.setID(id2_)
-                    self.awaiting_answers[id2_] = (BLOCKING_ARRIVED, )
-                    self.connection.send(iq)
-
-                if self.continue_connect_info and self.continue_connect_info[0]\
-                == 'invisible':
-                    # Trying to login as invisible but privacy list not
-                    # supported
-                    self.disconnect(on_purpose=True)
-                    gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
-                        show='offline'))
-                    gajim.nec.push_incoming_event(InformationEvent(None,
-                        conn=self, level='error', pri_txt=_('Invisibility not '
-                        'supported'), sec_txt=_('Account %s doesn\'t support '
-                        'invisibility.') % self.name))
-                    return
-            # Ask metacontacts before roster
-            self.get_metacontacts()
+                # That should never happen, but as it's blocking in the
+                # connection process, we don't take the risk
+                self.privacy_rules_supported = False
+                self._continue_connection_request_privacy()
         elif self.awaiting_answers[id_][0] == BLOCKING_ARRIVED:
+            del self.awaiting_answers[id_]
             if iq_obj.getType() == 'result':
                 list_node = iq_obj.getTag('blocklist')
                 if not list_node:
@@ -603,6 +593,7 @@ class ConnectionVcard:
                 for i in list_node.iterTags('item'):
                     self.blocked_contacts.append(i.getAttr('jid'))
         elif self.awaiting_answers[id_][0] == PEP_CONFIG:
+            del self.awaiting_answers[id_]
             if iq_obj.getType() == 'error':
                 return
             if not iq_obj.getTag('pubsub'):
@@ -617,71 +608,6 @@ class ConnectionVcard:
                 gajim.nec.push_incoming_event(PEPConfigReceivedEvent(None,
                     conn=self, node=node, form=form))
 
-        elif self.awaiting_answers[id_][0] == ARCHIVING_COLLECTIONS_ARRIVED:
-            # TODO
-            print 'ARCHIVING_COLLECTIONS_ARRIVED'
-
-        elif self.awaiting_answers[id_][0] == ARCHIVING_COLLECTION_ARRIVED:
-            def save_if_not_exists(with_, nick, direction, tim, payload):
-                assert len(payload) == 1, 'got several archiving messages in' +\
-                    ' the same time %s' % ''.join(payload)
-                if payload[0].getName() == 'body':
-                    gajim.logger.save_if_not_exists(with_, direction, tim,
-                        msg=payload[0].getData(), nick=nick)
-                elif payload[0].getName() == 'message':
-                    print 'Not implemented'
-            chat = iq_obj.getTag('chat')
-            if chat:
-                with_ = chat.getAttr('with')
-                start_ = chat.getAttr('start')
-                tim = helpers.datetime_tuple(start_)
-                tim = timegm(tim)
-                nb = 0
-                for element in chat.getChildren():
-                    try:
-                        secs = int(element.getAttr('secs'))
-                    except TypeError:
-                        secs = 0
-                    if secs:
-                        tim += secs
-                    nick = element.getAttr('name')
-                    if element.getName() == 'from':
-                        save_if_not_exists(with_, nick, 'from', localtime(tim),
-                            element.getPayload())
-                        nb += 1
-                    if element.getName() == 'to':
-                        save_if_not_exists(with_, nick, 'to', localtime(tim),
-                            element.getPayload())
-                        nb += 1
-                set_ = chat.getTag('set')
-                first = set_.getTag('first')
-                if first:
-                    try:
-                        index = int(first.getAttr('index'))
-                    except TypeError:
-                        index = 0
-                try:
-                    count = int(set_.getTagData('count'))
-                except TypeError:
-                    count = 0
-                if count > index + nb:
-                    # Request the next page
-                    after = element.getTagData('last')
-                    self.request_collection_page(with_, start_, after=after)
-
-        elif self.awaiting_answers[id_][0] == ARCHIVING_MODIFICATIONS_ARRIVED:
-            modified = iq_obj.getTag('modified')
-            if modified:
-                for element in modified.getChildren():
-                    if element.getName() == 'changed':
-                        with_ = element.getAttr('with')
-                        start_ = element.getAttr('start')
-                        self.request_collection_page(with_, start_)
-                    #elif element.getName() == 'removed':
-                        # do nothing
-
-        del self.awaiting_answers[id_]
-
     def _vCardCB(self, con, vc):
         """
         Called when we receive a vCard Parse the vCard and send it to plugins
@@ -968,6 +894,8 @@ class ConnectionHandlersBase:
             self._nec_gc_presence_received)
         gajim.ged.register_event_handler('message-received', ged.CORE,
             self._nec_message_received)
+        gajim.ged.register_event_handler('mam-message-received', ged.CORE,
+            self._nec_mam_message_received)
         gajim.ged.register_event_handler('decrypted-message-received', ged.CORE,
             self._nec_decrypted_message_received)
 
@@ -980,6 +908,8 @@ class ConnectionHandlersBase:
             self._nec_gc_presence_received)
         gajim.ged.remove_event_handler('message-received', ged.CORE,
             self._nec_message_received)
+        gajim.ged.remove_event_handler('mam-message-received', ged.CORE,
+            self._nec_mam_message_received)
         gajim.ged.remove_event_handler('decrypted-message-received', ged.CORE,
             self._nec_decrypted_message_received)
 
@@ -1192,14 +1122,46 @@ class ConnectionHandlersBase:
         gajim.nec.push_incoming_event(DecryptedMessageReceivedEvent(None,
             conn=self, msg_obj=obj))
 
+    def _nec_mam_message_received(self, obj):
+        if obj.conn.name != self.name:
+            return
+        if obj.enc_tag and self.USE_GPG:
+            encmsg = obj.enc_tag.getData()
+
+            keyID = gajim.config.get_per('accounts', self.name, 'keyid')
+            if keyID:
+                self.gpg_messages_to_decrypt.append([encmsg, keyID, obj])
+                if len(self.gpg_messages_to_decrypt) == 1:
+                    gajim.thread_interface(self.decrypt_thread, [encmsg, keyID,
+                        obj], self._on_mam_message_decrypted, [obj])
+                return
+        gajim.nec.push_incoming_event(MamDecryptedMessageReceivedEvent(None,
+            conn=self, msg_obj=obj))
+
     def _on_message_decrypted(self, output, obj):
         if len(self.gpg_messages_to_decrypt):
             encmsg, keyID, obj2 = self.gpg_messages_to_decrypt[0]
+            if type(obj2) == MessageReceivedEvent:
+                cb = self._on_message_decrypted
+            else:
+                cb = self._on_mam_message_decrypted
             gajim.thread_interface(self.decrypt_thread, [encmsg, keyID, obj2],
-                self._on_message_decrypted, [obj2])
+                cb, [obj2])
         gajim.nec.push_incoming_event(DecryptedMessageReceivedEvent(None,
             conn=self, msg_obj=obj))
 
+    def _on_mam_message_decrypted(self, output, obj):
+        if len(self.gpg_messages_to_decrypt):
+            encmsg, keyID, obj2 = self.gpg_messages_to_decrypt[0]
+            if type(obj2) == MessageReceivedEvent:
+                cb = self._on_message_decrypted
+            else:
+                cb = self._on_mam_message_decrypted
+            gajim.thread_interface(self.decrypt_thread, [encmsg, keyID, obj2],
+                cb, [obj2])
+        gajim.nec.push_incoming_event(MamDecryptedMessageReceivedEvent(None,
+            conn=self, msg_obj=obj))
+
     def _nec_decrypted_message_received(self, obj):
         if obj.conn.name != self.name:
             return
@@ -1423,13 +1385,14 @@ class ConnectionHandlersBase:
 
         return sess
 
-class ConnectionHandlers(ConnectionArchive, ConnectionVcard,
-ConnectionSocks5Bytestream, ConnectionDisco, ConnectionCommands,
-ConnectionPubSub, ConnectionPEP, ConnectionCaps, ConnectionHandlersBase,
-ConnectionJingle, ConnectionIBBytestream):
+class ConnectionHandlers(ConnectionArchive136, ConnectionArchive313,
+ConnectionVcard, ConnectionSocks5Bytestream, ConnectionDisco,
+ConnectionCommands, ConnectionPubSub, ConnectionPEP, ConnectionCaps,
+ConnectionHandlersBase, ConnectionJingle, ConnectionIBBytestream):
     def __init__(self):
         global HAS_IDLE
-        ConnectionArchive.__init__(self)
+        ConnectionArchive136.__init__(self)
+        ConnectionArchive313.__init__(self)
         ConnectionVcard.__init__(self)
         ConnectionSocks5Bytestream.__init__(self)
         ConnectionIBBytestream.__init__(self)
@@ -1529,7 +1492,8 @@ ConnectionJingle, ConnectionIBBytestream):
     def cleanup(self):
         ConnectionHandlersBase.cleanup(self)
         ConnectionCaps.cleanup(self)
-        ConnectionArchive.cleanup(self)
+        ConnectionArchive136.cleanup(self)
+        ConnectionArchive313.cleanup(self)
         ConnectionPubSub.cleanup(self)
         gajim.ged.remove_event_handler('http-auth-received', ged.CORE,
             self._nec_http_auth_received)
@@ -1677,7 +1641,7 @@ ConnectionJingle, ConnectionIBBytestream):
 
     def seclabel_catalogue_request(self, to, callback):
         if to not in self.seclabel_catalogues:
-            self.seclabel_catalogues[to] = [[], None, None]
+            self.seclabel_catalogues[to] = [[], None, None, None]
         self.seclabel_catalogues[to][0].append(callback)
 
     def _rosterSetCB(self, con, iq_obj):
