@@ -99,7 +99,7 @@ class Contact(CommonContact):
             our_chatstate, chatstate, client_caps=client_caps)
 
         self.contact_name = '' # nick choosen by contact
-        self.groups = [i for i in set(groups)] # filter duplicate values
+        self.groups = [i if i else _('General') for i in set(groups)] # filter duplicate values
 
         self.sub = sub
         self.ask = ask
@@ -836,6 +836,8 @@ class MetacontactManager():
             # we want one nearby_family per account
             nearby_family = [data for data in family if account == data['account']]
 
+        if not nearby_family:
+            return (None, None, None)
         big_brother_data = self._get_metacontacts_big_brother(nearby_family)
         big_brother_jid = big_brother_data['jid']
         big_brother_account = big_brother_data['account']
