@@ -149,7 +149,9 @@ class SocksQueue:
                 socks5obj = Socks5SenderClient(self.idlequeue, idx,
                     self, _sock=None,host=str(streamhost['host']),
                     port=int(streamhost['port']),fingerprint=fp,
-                    connected=False, file_props=file_props)
+                    connected=False, file_props=file_props, 
+                    initiator=streamhost['initiator'], 
+                    target=streamhost['target'])
                 socks5obj.streamhost = streamhost
                 self.add_sockobj(account, socks5obj)
 
@@ -914,7 +916,7 @@ class Socks5Sender(IdleObject):
     Class for sending file to socket over socks5
     """
     def __init__(self, idlequeue, sock_hash, parent, _sock, host=None,
-    port=None, fingerprint = None, connected=True, file_props={}):
+    port=None, fingerprint = None, connected=True, file_props=None):
         self.fingerprint = fingerprint
         self.queue_idx = sock_hash
         self.queue = parent
@@ -1321,8 +1323,9 @@ class Socks5Client(Socks5):
 class Socks5SenderClient(Socks5Client, Socks5Sender):
 
     def __init__(self, idlequeue, sock_hash, parent,_sock, host=None,
-    port=None, fingerprint = None, connected=True, file_props={}):
-        Socks5Client.__init__(self, idlequeue, host, port, None, None,
+    port=None, fingerprint = None, connected=True, file_props=None,
+    initiator=None, target=None):
+        Socks5Client.__init__(self, idlequeue, host, port, initiator, target,
             file_props.sid)
         Socks5Sender.__init__(self,idlequeue, sock_hash, parent,_sock,
             host, port, fingerprint , connected, file_props)
@@ -1331,7 +1334,7 @@ class Socks5SenderClient(Socks5Client, Socks5Sender):
 class Socks5SenderServer(Socks5Server, Socks5Sender):
 
     def __init__(self, idlequeue, sock_hash, parent,_sock, host=None,
-    port=None, fingerprint = None, connected=True, file_props={}):
+    port=None, fingerprint = None, connected=True, file_props=None):
         Socks5Server.__init__(self, idlequeue, host, port, None, None,
             file_props.sid)
         Socks5Sender.__init__(self,idlequeue, sock_hash, parent, _sock,
@@ -1339,7 +1342,7 @@ class Socks5SenderServer(Socks5Server, Socks5Sender):
 
 
 class Socks5ReceiverClient(Socks5Client, Socks5Receiver):
-    def __init__(self, idlequeue, streamhost, sid, file_props = None,
+    def __init__(self, idlequeue, streamhost, sid, file_props=None,
     fingerprint=None):
         Socks5Client.__init__(self, idlequeue, streamhost['host'],
             int(streamhost['port']), streamhost['initiator'],
@@ -1350,7 +1353,7 @@ class Socks5ReceiverClient(Socks5Client, Socks5Receiver):
 
 class Socks5ReceiverServer(Socks5Server, Socks5Receiver):
 
-    def __init__(self, idlequeue, streamhost, sid, file_props = None,
+    def __init__(self, idlequeue, streamhost, sid, file_props=None,
     fingerprint=None):
         Socks5Server.__init__(self, idlequeue, streamhost['host'],
             int(streamhost['port']), streamhost['initiator'],
