@@ -17,33 +17,31 @@
 ## along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 ##
 
-C_NAME, C_DOMAIN, C_INTERFACE, C_PROTOCOL, C_HOST, \
-C_ADDRESS, C_PORT, C_BARE_NAME, C_TXT = range(9)
+C_NAME, C_DOMAIN, C_RESOLVED_INFO, C_BARE_NAME, C_TXT = range(5)
+C_RI_INTERFACE, C_RI_PROTOCOL, C_RI_HOST, C_RI_APROTOCOL, C_RI_ADDRESS, C_RI_PORT = range(6)
 
 def test_avahi():
-	try:
-		import avahi
-	except ImportError:
-		return False
-	return True
+    try:
+        import avahi
+    except ImportError:
+        return False
+    return True
 
 def test_bonjour():
-	try:
-		import pybonjour
-	except ImportError:
-		return False
-	except WindowsError:
-		return False
-	return True
+    try:
+        import pybonjour
+    except ImportError:
+        return False
+    except WindowsError:
+        return False
+    return True
 
 def test_zeroconf():
-	return test_avahi() or test_bonjour()
+    return test_avahi() or test_bonjour()
 
 if test_avahi():
-	from common.zeroconf import zeroconf_avahi
-	Zeroconf = zeroconf_avahi.Zeroconf
+    from common.zeroconf import zeroconf_avahi
+    Zeroconf = zeroconf_avahi.Zeroconf
 elif test_bonjour():
-	from common.zeroconf import zeroconf_bonjour
-	Zeroconf = zeroconf_bonjour.Zeroconf
-
-# vim: se ts=3:
\ No newline at end of file
+    from common.zeroconf import zeroconf_bonjour
+    Zeroconf = zeroconf_bonjour.Zeroconf
