@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/dataforms_widget.py
 ##
-## Copyright (C) 2003-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2006 Tomasz Melcer <liori AT exroot.org>
 ## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
 ##
@@ -52,6 +52,7 @@ class DataFormWidget(gtk.Alignment, object):
         gtk.Alignment.__init__(self, xscale=1.0, yscale=1.0)
 
         self._data_form = None
+        self.selectable = False
 
         self.xml = gtkgui_helpers.get_gtk_builder('data_form_window.ui',
                 'data_form_vbox')
@@ -77,7 +78,7 @@ class DataFormWidget(gtk.Alignment, object):
 
     def set_data_form(self, dataform):
         """
-        Set the data form (xmpp.DataForm) displayed in widget
+        Set the data form (nbxmpp.DataForm) displayed in widget
         """
         assert isinstance(dataform, dataforms.DataForm)
 
@@ -147,7 +148,8 @@ class DataFormWidget(gtk.Alignment, object):
 
         self.clean_data_form()
 
-        self.singleform = SingleForm(self._data_form)
+        self.singleform = SingleForm(self._data_form,
+            selectable=self.selectable)
         def _on_validated(widget):
             self.emit('validated')
         self.singleform.connect('validated', _on_validated)
@@ -183,7 +185,7 @@ class DataFormWidget(gtk.Alignment, object):
             # note: we store also text-private and hidden fields,
             # we just do not display them.
             # TODO: boolean fields
-            #elif field.type=='boolean': fieldtypes.append(bool)
+            #elif field.type_=='boolean': fieldtypes.append(bool)
             fieldtypes.append(str)
             fieldvars.append(field.var)
 
@@ -193,8 +195,9 @@ class DataFormWidget(gtk.Alignment, object):
         for item in self._data_form.iter_records():
             iter_ = self.multiplemodel.append()
             for field in item.iter_fields():
-                self.multiplemodel.set_value(iter_, fieldvars.index(field.var),
-                        field.value)
+                if field.var in fieldvars:
+                    self.multiplemodel.set_value(iter_,
+                        fieldvars.index(field.var), field.value)
 
         # constructing columns...
         for field, counter in zip(self._data_form.reported.iter_fields(),
@@ -212,7 +215,7 @@ class DataFormWidget(gtk.Alignment, object):
 
         self.clean_data_form = self.clean_multiple_data_form
 
-        readwrite = self._data_form.type != 'result'
+        readwrite = self._data_form.type_ != 'result'
         if not readwrite:
             self.buttons_vbox.set_no_show_all(True)
             self.buttons_vbox.hide()
@@ -313,7 +316,7 @@ class SingleForm(gtk.Table, object):
         validated = (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
     )
 
-    def __init__(self, dataform):
+    def __init__(self, dataform, selectable=False):
         assert isinstance(dataform, dataforms.SimpleDataForm)
 
         gtk.Table.__init__(self)
@@ -340,25 +343,25 @@ class SingleForm(gtk.Table, object):
         linecounter = 0
 
         # is the form changeable?
-        readwrite = dataform.type != 'result'
+        readwrite = dataform.type_ != 'result'
 
         # for each field...
         for field in self._data_form.iter_fields():
-            if field.type == 'hidden': continue
+            if field.type_ == 'hidden': continue
 
             commonlabel = True
             commonlabelcenter = False
             commonwidget = True
             widget = None
 
-            if field.type == 'boolean':
+            if field.type_ == 'boolean':
                 commonlabelcenter = True
                 widget = gtk.CheckButton()
                 widget.connect('toggled', self.on_boolean_checkbutton_toggled,
                         field)
                 widget.set_active(field.value)
 
-            elif field.type == 'fixed':
+            elif field.type_ == 'fixed':
                 leftattach = 1
                 rightattach = 2
                 if field.label is None:
@@ -367,11 +370,12 @@ class SingleForm(gtk.Table, object):
 
                 commonwidget = False
                 widget = gtk.Label(field.value)
+                widget.set_property('selectable', selectable)
                 widget.set_line_wrap(True)
                 self.attach(widget, leftattach, rightattach, linecounter,
                         linecounter+1, xoptions=gtk.FILL, yoptions=gtk.FILL)
 
-            elif field.type == 'list-single':
+            elif field.type_ == 'list-single':
                 # TODO: What if we have radio buttons and non-required field?
                 # TODO: We cannot deactivate them all...
                 if len(field.options) < 6:
@@ -405,7 +409,7 @@ class SingleForm(gtk.Table, object):
                     widget.connect('changed', on_list_single_combobox_changed, field)
                 widget.set_sensitive(readwrite)
 
-            elif field.type == 'list-multi':
+            elif field.type_ == 'list-multi':
                 # TODO: When more than few choices, make a list
                 if len(field.options) < 6:
                     # 5 option max: show checkbutton
@@ -435,12 +439,12 @@ class SingleForm(gtk.Table, object):
                             on_list_multi_treeview_changed, field)
                     tv.set_sensitive(readwrite)
 
-            elif field.type == 'jid-single':
+            elif field.type_ == 'jid-single':
                 widget = gtk.Entry()
                 widget.connect('changed', self.on_text_single_entry_changed, field)
                 widget.set_text(field.value)
 
-            elif field.type == 'jid-multi':
+            elif field.type_ == 'jid-multi':
                 commonwidget = False
 
                 xml = gtkgui_helpers.get_gtk_builder('data_form_window.ui',
@@ -489,14 +493,14 @@ class SingleForm(gtk.Table, object):
 
                 del xml
 
-            elif field.type == 'text-private':
+            elif field.type_ == 'text-private':
                 commonlabelcenter = True
                 widget = gtk.Entry()
                 widget.connect('changed', self.on_text_single_entry_changed, field)
                 widget.set_visibility(False)
                 widget.set_text(field.value)
 
-            elif field.type == 'text-multi':
+            elif field.type_ == 'text-multi':
                 # TODO: bigger text view
                 commonwidget = False
 
@@ -505,7 +509,13 @@ class SingleForm(gtk.Table, object):
                 textwidget.get_buffer().connect('changed',
                         self.on_text_multi_textbuffer_changed, field)
                 textwidget.get_buffer().set_text(field.value)
-                textwidget.set_sensitive(readwrite)
+                if readwrite:
+                    textwidget.set_sensitive(True)
+                else:
+                    if selectable:
+                        textwidget.set_editable(True)
+                    else:
+                        textwidget.set_sensitive(False)
 
                 widget = gtk.ScrolledWindow()
                 widget.add(textwidget)
@@ -514,7 +524,7 @@ class SingleForm(gtk.Table, object):
                 self.attach(widget, 1, 2, linecounter, linecounter+1)
 
             else:
-                # field.type == 'text-single' or field.type is nonstandard:
+                # field.type_ == 'text-single' or field.type_ is nonstandard:
                 # JEP says that if we don't understand some type, we
                 # should handle it as text-single
                 commonlabelcenter = True
@@ -534,6 +544,7 @@ class SingleForm(gtk.Table, object):
                 else:
                     commonwidget=False
                     widget = gtk.Label(field.value)
+                    widget.set_property('selectable', selectable)
                     widget.set_sensitive(True)
                     widget.set_alignment(0.0, 0.5)
                     widget=decorate_with_tooltip(widget, field)
@@ -575,7 +586,12 @@ class SingleForm(gtk.Table, object):
                 widget = decorate_with_tooltip(widget, field)
                 self.attach(widget, 1, 2, linecounter, linecounter+1,
                         yoptions=gtk.FILL)
-            widget.show_all()
+
+            if field.required:
+                label = gtk.Label('*')
+                label.set_tooltip_text(_('This field is required'))
+                self.attach(label, 2, 3, linecounter, linecounter+1, xoptions=0,
+                    yoptions=0)
 
             linecounter+=1
         if self.get_property('visible'):
