@@ -69,7 +69,7 @@ if os.name == 'nt':
         pass
 
 HAS_NBXMPP=True
-MIN_NBXMPP_VER = "0.5.1"
+MIN_NBXMPP_VER = "0.5.3"
 try:
     import nbxmpp
 except ImportError:
@@ -96,7 +96,7 @@ demandimport.ignore += ['gobject._gobject', 'libasyncns', 'i18n',
     'logging.NullHandler', 'dbus.service', 'OpenSSL.SSL', 'OpenSSL.crypto',
     'common.sleepy', 'DLFCN', 'dl', 'xml.sax', 'xml.sax.handler', 'ic',
     'Crypto.PublicKey', 'IPython', 'contextlib', 'imp', 'gst.interfaces',
-    'monotonic', 'gtkexcepthook']
+    'libxml2', 'libxml2mod', 'monotonic', 'gtkexcepthook']
 
 if os.name == 'nt':
     import locale
