@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/history_window.py
 ##
-## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005 Vincent Hanquez <tab AT snarc.org>
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
@@ -69,13 +69,14 @@ class HistoryWindow:
         self.calendar = xml.get_object('calendar')
         scrolledwindow = xml.get_object('scrolledwindow')
         self.history_textview = conversation_textview.ConversationTextview(
-                account, used_in_history_window = True)
+            account, used_in_history_window = True)
         scrolledwindow.add(self.history_textview.tv)
         self.history_buffer = self.history_textview.tv.get_buffer()
         self.history_buffer.create_tag('highlight', background = 'yellow')
         self.checkbutton = xml.get_object('log_history_checkbutton')
         self.checkbutton.connect('toggled',
-                self.on_log_history_checkbutton_toggled)
+            self.on_log_history_checkbutton_toggled)
+        self.show_status_checkbutton = xml.get_object('show_status_checkbutton')
         self.query_entry = xml.get_object('query_entry')
         self.query_combobox = xml.get_object('query_combobox')
         self.jid_entry = self.query_combobox.child
@@ -83,6 +84,7 @@ class HistoryWindow:
         self.query_combobox.set_active(0)
         self.results_treeview = xml.get_object('results_treeview')
         self.results_window = xml.get_object('results_scrolledwindow')
+        self.search_in_date = xml.get_object('search_in_date')
 
         # contact_name, date, message, time
         model = gtk.ListStore(str, str, str, str, str)
@@ -125,11 +127,11 @@ class HistoryWindow:
             self._load_history(None)
 
         gtkgui_helpers.resize_window(self.window,
-                gajim.config.get('history_window_width'),
-                gajim.config.get('history_window_height'))
+            gajim.config.get('history_window_width'),
+            gajim.config.get('history_window_height'))
         gtkgui_helpers.move_window(self.window,
-                gajim.config.get('history_window_x-position'),
-                gajim.config.get('history_window_y-position'))
+            gajim.config.get('history_window_x-position'),
+            gajim.config.get('history_window_y-position'))
 
         xml.connect_signals(self)
         self.window.show_all()
@@ -201,9 +203,9 @@ class HistoryWindow:
 
             liststore.append((pix, completed))
             self.completion_dict[key] = (info_jid, info_acc, info_name,
-                    info_completion)
+                info_completion)
             self.completion_dict[completed] = (info_jid, info_acc,
-                    info_name, info_completion)
+                info_name, info_completion)
             if key == actual_jid:
                 self._load_history(info_jid, info_acc)
             yield True
@@ -257,7 +259,8 @@ class HistoryWindow:
         """
         if jid_or_name and jid_or_name in self.completion_dict:
         # a full qualified jid or a contact name was entered
-            info_jid, info_account, info_name, info_completion = self.completion_dict[jid_or_name]
+            info_jid, info_account, info_name, info_completion = \
+                self.completion_dict[jid_or_name]
             self.jids_to_search = [info_jid]
             self.jid = info_jid
 
@@ -273,14 +276,14 @@ class HistoryWindow:
             else:
                 # Are log disabled for account ?
                 if self.account in gajim.config.get_per('accounts', self.account,
-                        'no_log_for').split(' '):
+                'no_log_for').split(' '):
                     self.checkbutton.set_active(False)
                     self.checkbutton.set_sensitive(False)
                 else:
                     # Are log disabled for jid ?
                     log = True
                     if self.jid in gajim.config.get_per('accounts', self.account,
-                            'no_log_for').split(' '):
+                    'no_log_for').split(' '):
                         log = False
                     self.checkbutton.set_active(log)
                     self.checkbutton.set_sensitive(True)
@@ -289,8 +292,8 @@ class HistoryWindow:
 
             # select logs for last date we have logs with contact
             self.calendar.set_sensitive(True)
-            last_log = \
-                    gajim.logger.get_last_date_that_has_logs(self.jid, self.account)
+            last_log = gajim.logger.get_last_date_that_has_logs(self.jid,
+                self.account)
 
             date = time.localtime(last_log)
 
@@ -326,7 +329,7 @@ class HistoryWindow:
     def on_calendar_day_selected(self, widget):
         if not self.jid:
             return
-        year, month, day = widget.get_date() # integers
+        year, month, day = self.calendar.get_date() # integers
         month = gtkgui_helpers.make_gtk_month_python_month(month)
         self._add_lines_for_date(year, month, day)
 
@@ -379,13 +382,18 @@ class HistoryWindow:
         """
         self.history_buffer.set_text('') # clear the buffer first
         self.last_time_printout = 0
+        show_status = self.show_status_checkbutton.get_active()
 
-        lines = gajim.logger.get_conversation_for_date(self.jid, year, month, day, self.account)
+        lines = gajim.logger.get_conversation_for_date(self.jid, year, month,
+            day, self.account)
         # lines holds list with tupples that have:
         # contact_name, time, kind, show, message
         for line in lines:
             # line[0] is contact_name, line[1] is time of message
             # line[2] is kind, line[3] is show, line[4] is message
+            if not show_status and line[2] in (constants.KIND_GCSTATUS,
+            constants.KIND_STATUS):
+                continue
             self._add_new_line(line[0], line[1], line[2], line[3], line[4],
                     line[5])
 
@@ -394,7 +402,7 @@ class HistoryWindow:
         Add a new line in textbuffer
         """
         if not message and kind not in (constants.KIND_STATUS,
-                constants.KIND_GCSTATUS):
+        constants.KIND_GCSTATUS):
             return
         buf = self.history_buffer
         end_iter = buf.get_end_iter()
@@ -406,13 +414,13 @@ class HistoryWindow:
             buf.insert(end_iter, tim) # add time
         elif gajim.config.get('print_time') == 'sometimes':
             every_foo_seconds = 60 * gajim.config.get(
-                    'print_ichat_every_foo_minutes')
+                'print_ichat_every_foo_minutes')
             seconds_passed = tim - self.last_time_printout
             if seconds_passed > every_foo_seconds:
                 self.last_time_printout = tim
                 tim = time.strftime('%X ', time.localtime(float(tim)))
                 buf.insert_with_tags_by_name(end_iter, tim + '\n',
-                        'time_sometimes')
+                    'time_sometimes')
 
         tag_name = ''
         tag_msg = ''
@@ -441,11 +449,11 @@ class HistoryWindow:
             # message here (if not None) is status message
             if message:
                 message = _('%(nick)s is now %(status)s: %(status_msg)s') %\
-                        {'nick': contact_name, 'status': helpers.get_uf_show(show),
-                        'status_msg': message }
+                    {'nick': contact_name, 'status': helpers.get_uf_show(show),
+                    'status_msg': message }
             else:
-                message = _('%(nick)s is now %(status)s') % {'nick': contact_name,
-                        'status': helpers.get_uf_show(show) }
+                message = _('%(nick)s is now %(status)s') % {
+                    'nick': contact_name, 'status': helpers.get_uf_show(show) }
             tag_msg = 'status'
         else: # 'status'
             # message here (if not None) is status message
@@ -456,10 +464,10 @@ class HistoryWindow:
                     message = _('Error')
             elif message:
                 message = _('Status is now: %(status)s: %(status_msg)s') % \
-                        {'status': helpers.get_uf_show(show), 'status_msg': message}
+                    {'status': helpers.get_uf_show(show), 'status_msg': message}
             else:
                 message = _('Status is now: %(status)s') % { 'status':
-                        helpers.get_uf_show(show) }
+                    helpers.get_uf_show(show) }
             tag_msg = 'status'
 
         if message.startswith('/me ') or message.startswith('/me\n'):
@@ -484,7 +492,7 @@ class HistoryWindow:
 
         if tag_msg:
             self.history_textview.print_real_text(message, [tag_msg],
-                    name=contact_name, xhtml=xhtml)
+                name=contact_name, xhtml=xhtml)
         else:
             self.history_textview.print_real_text(message, name=contact_name,
                 xhtml=xhtml)
@@ -505,19 +513,30 @@ class HistoryWindow:
         for jid in self.jids_to_search:
             account = self.completion_dict[jid][C_INFO_ACCOUNT]
             if account is None:
-                # We do not know an account. This can only happen if the contact is offine,
-                # or if we browse a groupchat history. The account is not needed, a dummy can
-                # be set.
-                # This may leed to wrong self nick in the displayed history (Uggh!)
+                # We do not know an account. This can only happen if the contact
+                # is offine, or if we browse a groupchat history. The account is
+                # not needed, a dummy can be set.
+                # This may leed to wrong self nick in the displayed history
+                # (Uggh!)
                 account = gajim.contacts.get_accounts()[0]
 
+            year, month, day = False, False, False
+            if self.search_in_date.get_active():
+                year, month, day = self.calendar.get_date() # integers
+                month = gtkgui_helpers.make_gtk_month_python_month(month)
+
+            show_status = self.show_status_checkbutton.get_active()
+
             # contact_name, time, kind, show, message, subject
-            results = gajim.logger.get_search_results_for_query(
-                                    jid, text, account)
+            results = gajim.logger.get_search_results_for_query(jid, text,
+                account, year, month, day)
             #FIXME:
             # add "subject:  | message: " in message column if kind is single
             # also do we need show at all? (we do not search on subject)
             for row in results:
+                if not show_status and row[2] in (constants.KIND_GCSTATUS,
+                constants.KIND_STATUS):
+                    continue
                 contact_name = row[0]
                 if not contact_name:
                     kind = row[2]
@@ -551,7 +570,7 @@ class HistoryWindow:
             self.query_entry.set_sensitive(True)
             self.query_entry.grab_focus()
             self.jids_to_search = (jid for jid in gajim.logger.get_jids_in_db()
-                            if gajim.logger.jid_is_room_jid(jid))
+                if gajim.logger.jid_is_room_jid(jid))
         if self.query_combobox.get_active() == 2:
             # All Chat Histories
             self.query_entry.set_sensitive(True)
@@ -596,25 +615,28 @@ class HistoryWindow:
         """
         start_iter = self.history_buffer.get_start_iter()
         local_time = time.localtime(float(unix_time))
-        tim = time.strftime('%X', local_time)
+        timestamp_str = gajim.config.get('time_stamp')
+        timestamp_str = helpers.from_one_line(timestamp_str)
+        tim = time.strftime(timestamp_str, local_time)
         result = start_iter.forward_search(tim, gtk.TEXT_SEARCH_VISIBLE_ONLY,
                 None)
         if result is not None:
             match_start_iter, match_end_iter = result
-            match_start_iter.backward_char() # include '[' or other character before time
+            # include '[' or other character before time
+            match_start_iter.backward_char()
             match_end_iter.forward_line() # highlight all message not just time
             self.history_buffer.apply_tag_by_name('highlight', match_start_iter,
-                    match_end_iter)
+                match_end_iter)
 
             match_start_mark = self.history_buffer.create_mark('match_start',
-                    match_start_iter, True)
+                match_start_iter, True)
             self.history_textview.tv.scroll_to_mark(match_start_mark, 0, True)
 
     def on_log_history_checkbutton_toggled(self, widget):
         # log conversation history?
         oldlog = True
         no_log_for = gajim.config.get_per('accounts', self.account,
-                'no_log_for').split()
+            'no_log_for').split()
         if self.jid in no_log_for:
             oldlog = False
         log = widget.get_active()
@@ -624,7 +646,11 @@ class HistoryWindow:
             no_log_for.remove(self.jid)
         if oldlog != log:
             gajim.config.set_per('accounts', self.account, 'no_log_for',
-                    ' '.join(no_log_for))
+                ' '.join(no_log_for))
+
+    def on_show_status_checkbutton_toggled(self, widget):
+        # reload logs
+        self.on_calendar_day_selected(None)
 
     def open_history(self, jid, account):
         """
@@ -648,5 +674,3 @@ class HistoryWindow:
         gajim.config.set('history_window_y-position', y)
         gajim.config.set('history_window_width', width);
         gajim.config.set('history_window_height', height);
-
-        gajim.interface.save_config()
