@@ -489,10 +489,10 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
             tag.title = title
         return tag
 
-    def _update_img(self, (mem, alt), attrs, img_mark):
+    def _update_img(self, (mem, alt), attrs, img_mark, tags):
         '''Callback function called after the function helpers.download_image.
         '''
-        self._process_img(attrs, (mem, alt, img_mark))
+        self._process_img(attrs, (mem, alt, img_mark, tags))
 
     def _process_img(self, attrs, loaded=None):
         '''Process a img tag.
@@ -501,6 +501,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         update = False
         pixbuf = None
         replace_mark = None
+        replace_tags = None
 
         try:
             if attrs['src'].startswith('data:image/'):
@@ -509,14 +510,14 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
                 img = attrs['src'].split(',')[1]
                 mem = base64.standard_b64decode(urllib2.unquote(img))
             elif loaded is not None:
-                (mem, alt, replace_mark) = loaded
+                (mem, alt, replace_mark, replace_tags) = loaded
                 update = True
             else:
                 if self.conv_textview:
                     img_mark = self.textbuf.create_mark(None, self.iter, True)
                     gajim.thread_interface(helpers.download_image, [
                         self.conv_textview.account, attrs], self._update_img,
-                        [attrs, img_mark])
+                        [attrs, img_mark, self._get_style_tags()])
                     alt = attrs.get('alt', '')
                     if alt:
                         alt += '\n'
@@ -574,7 +575,10 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
                 self.textbuf.delete(working_iter, next_iter)
                 self.textbuf.delete_mark(replace_mark)
             if pixbuf is not None:
-                tags = self._get_style_tags()
+                if replace_mark:
+                    tags = replace_tags
+                else:
+                    tags = self._get_style_tags()
                 if tags:
                     tmpmark = self.textbuf.create_mark(None, working_iter, True)
                 self.textbuf.insert_pixbuf(working_iter, pixbuf)
@@ -910,7 +914,7 @@ class HtmlTextView(gtk.TextView):
         dialogs.AddNewContactWindow(self.account, jid)
 
     def make_link_menu(self, event, kind, text):
-        from gtkgui_helpers import get_gtk_builder
+        from gtkgui_helpers import get_gtk_builder, load_icon
         xml = get_gtk_builder('chat_context_menu.ui')
         menu = xml.get_object('chat_context_menu')
         childs = menu.get_children()
@@ -927,7 +931,7 @@ class HtmlTextView(gtk.TextView):
         else: # It's a mail or a JID
             # load muc icon
             join_group_chat_menuitem = xml.get_object('join_group_chat_menuitem')
-            muc_icon = gtkgui_helpers.load_icon('muc_active')
+            muc_icon = load_icon('muc_active')
             if muc_icon:
                 join_group_chat_menuitem.set_image(muc_icon)
 
@@ -1271,4 +1275,7 @@ hhx4dbgYKAAA7' alt='Larry'/>
     w.set_default_size(400, 300)
     w.show_all()
     w.connect('destroy', lambda w: gtk.main_quit())
+    gtk.gdk.threads_init()
+    gtk.gdk.threads_enter()
     gtk.main()
+    gtk.gdk.threads_leave()
