@@ -311,7 +311,8 @@ class MessageWindow(object):
         # NOTE: we do not call set_control_active(True) since we don't know
         # whether the tab is the active one.
         self.show_title()
-        gobject.timeout_add(500, control.msg_textview.grab_focus)
+        if self.get_num_controls() == 1:
+            gobject.timeout_add(500, control.msg_textview.grab_focus)
 
     def on_tab_eventbox_button_press_event(self, widget, event, child):
         if event.button == 3: # right click
@@ -366,6 +367,10 @@ class MessageWindow(object):
                 # theme
                 if not gtk.settings_get_default().get_property(
                 'gtk-key-theme-name') == 'Emacs':
+                    if gajim.interface.msg_win_mgr.mode == \
+                    gajim.interface.msg_win_mgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+                        gajim.interface.roster.tree.grab_focus()
+                        return False
                     control._on_send_file_menuitem_activate(None)
                     return True
             elif control.type_id == message_control.TYPE_CHAT and \
@@ -384,8 +389,12 @@ class MessageWindow(object):
                 return True
             elif control.type_id == message_control.TYPE_GC and \
             keyval == gtk.keysyms.b: # CTRL + b
-                control._on_bookmark_room_menuitem_activate(None)
-                return True
+                # CTRL + b moves cursor one char backward when user uses Emacs
+                # theme
+                if not gtk.settings_get_default().get_property(
+                'gtk-key-theme-name') == 'Emacs':
+                    control._on_bookmark_room_menuitem_activate(None)
+                    return True
             # Tab switch bindings
             elif keyval == gtk.keysyms.F4: # CTRL + F4
                 self.remove_tab(control, self.CLOSE_CTRL_KEY)
@@ -511,7 +520,8 @@ class MessageWindow(object):
 
         if control.type_id == message_control.TYPE_GC:
             name = control.room_jid.split('@')[0]
-            urgent = control.attention_flag
+            urgent = control.attention_flag or \
+                gajim.config.get('notify_on_all_muc_messages')
         else:
             name = control.contact.get_shown_name()
             if control.resource:
